@@ -116,11 +116,25 @@ query_axfr(struct nsd *nsd, struct query *query, int wstats)
 			    && query->axfr_current_rrset->zone == query->axfr_zone)
 			{
 				while (query->axfr_current_rr < query->axfr_current_rrset->rr_count) {
+					size_t oldmaxlen = query->maxlen;
+					if(total_added == 0)
+						/* RR > 16K can be first RR */
+						query->maxlen = (query->tcp?TCP_MAX_MESSAGE_LEN:UDP_MAX_MESSAGE_LEN);
 					added = packet_encode_rr(
 						query,
 						query->axfr_current_domain,
 						&query->axfr_current_rrset->rrs[query->axfr_current_rr],
 						query->axfr_current_rrset->rrs[query->axfr_current_rr].ttl);
+					if(total_added == 0) {
+						query->maxlen = oldmaxlen;
+						if(query_overflow(query)) {
+							if(added) {
+								++total_added;
+								++query->axfr_current_rr;
+								goto return_answer;
+							}
+						}
+					}
 					if (!added)
 						goto return_answer;
 					++total_added;
@@ -174,12 +188,35 @@ static int axfr_ixfr_can_admit_query(struct nsd* nsd, struct query* q)
 	struct acl_options *acl = NULL;
 	struct zone_options* zone_opt;
 	zone_opt = zone_options_find(nsd->options, q->qname);
+	if(zone_opt && q->is_proxied && acl_check_incoming_block_proxy(
+		zone_opt->pattern->provide_xfr, q, &acl) == -1) {
+		/* the proxy address is blocked */
+		if (verbosity >= 2) {
+			char address[128], proxy[128];
+			addr2str(&q->client_addr, address, sizeof(address));
+			addr2str(&q->remote_addr, proxy, sizeof(proxy));
+			VERBOSITY(2, (LOG_INFO, "%s for %s from %s via proxy %s refused because of proxy, %s %s",
+				(q->qtype==TYPE_AXFR?"axfr":"ixfr"),
+				dname_to_string(q->qname, NULL),
+				address, proxy,
+				(acl?acl->ip_address_spec:"."),
+				(acl ? ( acl->nokey    ? "NOKEY"
+				      : acl->blocked  ? "BLOCKED"
+				      : acl->key_name )
+				    : "no acl matches")));
+		}
+		RCODE_SET(q->packet, RCODE_REFUSE);
+		/* RFC8914 - Extended DNS Errors
+		 * 4.19.  Extended DNS Error Code 18 - Prohibited */
+		q->edns.ede = EDE_PROHIBITED;
+		return 0;
+	}
 	if(!zone_opt ||
 	   acl_check_incoming(zone_opt->pattern->provide_xfr, q, &acl)==-1)
 	{
 		if (verbosity >= 2) {
 			char a[128];
-			addr2str(&q->addr, a, sizeof(a));
+			addr2str(&q->client_addr, a, sizeof(a));
 			VERBOSITY(2, (LOG_INFO, "%s for %s from %s refused, %s",
 				(q->qtype==TYPE_AXFR?"axfr":"ixfr"),
 				dname_to_string(q->qname, NULL), a, acl?"blocked":"no acl matches"));
@@ -202,7 +239,7 @@ static int axfr_ixfr_can_admit_query(struct nsd* nsd, struct query* q)
 		acl->ip_address_spec, acl->key_name?acl->key_name:"NOKEY"));
 	if (verbosity >= 1) {
 		char a[128];
-		addr2str(&q->addr, a, sizeof(a));
+		addr2str(&q->client_addr, a, sizeof(a));
 		VERBOSITY(1, (LOG_INFO, "%s for %s from %s",
 			(q->qtype==TYPE_AXFR?"axfr":"ixfr"),
 			dname_to_string(q->qname, NULL), a));
