@@ -59,12 +59,12 @@ void delete_zone_rrs(namedb_type* db, zone_type* zone);
 int delete_RR(namedb_type* db, const dname_type* dname,
 	uint16_t type, uint16_t klass,
 	buffer_type* packet, size_t rdatalen, zone_type *zone,
-	region_type* temp_region, struct udb_ptr* udbz, int* softfail);
+	region_type* temp_region, int* softfail);
 /* add an RR */
 int add_RR(namedb_type* db, const dname_type* dname,
 	uint16_t type, uint16_t klass, uint32_t ttl,
 	buffer_type* packet, size_t rdatalen, zone_type *zone,
-	struct udb_ptr* udbz, int* softfail);
+	int* softfail);
 
 enum soainfo_hint {
 	soainfo_ok,
@@ -90,8 +90,6 @@ struct task_list_d {
 		task_write_zonefiles,
 		/** set verbosity */
 		task_set_verbosity,
-		/** statistic info */
-		task_stat_info,
 		/** add a zone */
 		task_add_zone,
 		/** delete zone */
@@ -134,8 +132,6 @@ void task_clear(udb_base* udb);
 void task_new_soainfo(udb_base* udb, udb_ptr* last, struct zone* z, enum soainfo_hint hint);
 void task_new_expire(udb_base* udb, udb_ptr* last,
 	const struct dname* z, int expired);
-void* task_new_stat_info(udb_base* udb, udb_ptr* last, struct nsdst* stat,
-	size_t child_count);
 void task_new_check_zonefiles(udb_base* udb, udb_ptr* last,
 	const dname_type* zone);
 void task_new_write_zonefiles(udb_base* udb, udb_ptr* last,
