@@ -243,6 +243,7 @@ int dname_parse_wire(uint8_t* dname, const char* name)
 		}
 		*h = label_length;
 		h = p;
+		p++;
 	}
 
 	/* Add root label.  */
@@ -416,7 +417,7 @@ dname_to_string(const dname_type *dname, const dname_type *origin)
 		++src;
 		for (j = 0; j < len; ++j) {
 			uint8_t ch = *src++;
-			if (isalnum((unsigned char)ch) || ch == '-' || ch == '_') {
+			if (isalnum((unsigned char)ch) || ch == '-' || ch == '_' || ch == '*') {
 				*dst++ = ch;
 			} else if (ch == '.' || ch == '\\') {
 				*dst++ = '\\';
@@ -512,7 +513,7 @@ char* wirelabel2str(const uint8_t* label)
 	lablen = *label++;
 	while(lablen--) {
 		uint8_t ch = *label++;
-		if (isalnum((unsigned char)ch) || ch == '-' || ch == '_') {
+		if (isalnum((unsigned char)ch) || ch == '-' || ch == '_' || ch == '*') {
 			*p++ = ch;
 		} else if (ch == '.' || ch == '\\') {
 			*p++ = '\\';
