@@ -31,8 +31,12 @@ typedef struct xfrd_tcp_set xfrd_tcp_set_type;
  * A set of xfrd tcp connections.
  */
 struct xfrd_tcp_set {
-	/* tcp connections, each has packet and read/wr state */
-	struct xfrd_tcp_pipeline *tcp_state[XFRD_MAX_TCP];
+	/* tcp connections, array, each has packet and read/wr state */
+	struct xfrd_tcp_pipeline **tcp_state;
+	/* max number of tcp connections, size of tcp_state array */
+	int tcp_max;
+	/* max number of simultaneous connections on a tcp_pipeline */
+	int tcp_pipeline;
 	/* number of TCP connections in use. */
 	int tcp_count;
 	/* TCP timeout. */
@@ -74,8 +78,21 @@ struct xfrd_tcp {
 /* use illegal pointer value to denote skipped ID number.
  * if this does not work, we can allocate with malloc */
 #define TCP_NULL_SKIP ((struct xfrd_zone*)-1)
-/* the number of ID values (16 bits) for a pipeline */
-#define ID_PIPE_NUM 65536
+
+/**
+ * The per-id zone pointers, with TCP_NULL_SKIP or a zone pointer for the
+ * ID value.
+ */
+struct xfrd_tcp_pipeline_id {
+	/** rbtree node as first member, this is the key. */
+	rbnode_type node;
+	/** the ID of this member */
+	uint16_t id;
+	/** zone pointer or TCP_NULL_SKIP */
+	struct xfrd_zone* zone;
+	/** next free in free list */
+	struct xfrd_tcp_pipeline_id* next_free;
+};
 
 /**
  * The tcp pipeline key structure. By ip_len, ip, num_unused and unique by
@@ -138,23 +155,31 @@ struct xfrd_tcp_pipeline {
 	/* list of queries that want to send, first to get write event,
 	 * if NULL, no write event interest */
 	struct xfrd_zone* tcp_send_first, *tcp_send_last;
-	/* the unused and id arrays must be last in the structure */
-	/* per-ID number the queries that have this ID number, every
+
+	/* size of the id and unused arrays. */
+	int pipe_num;
+	/* list of free xfrd_tcp_pipeline_id nodes, these are not in the
+	 * zone_per_id tree. preallocated at pipe_num amount. */
+	struct xfrd_tcp_pipeline_id* pipe_id_free_list;
+	/* The xfrd_zone pointers, per id number.
+	 * The key is struct xfrd_tcp_pipeline_id.
+	 * per-ID number the queries that have this ID number, every
 	 * query owns one ID numbers (until it is done). NULL: unused
 	 * When a query is done but not all answer-packets have been
 	 * consumed for that ID number, the rest is skipped, this
 	 * is denoted with the pointer-value TCP_NULL_SKIP, the ids that
 	 * are skipped are not on the unused list.  They may be
 	 * removed once the last answer packet is skipped.
-	 * ID_PIPE_NUM-num_unused values in the id array are nonNULL (either
+	 * pipe_num-num_unused values are in the tree (either
 	 * a zone pointer or SKIP) */
-	struct xfrd_zone* id[ID_PIPE_NUM];
-	/* unused ID numbers; the first part of the array contains the IDs */
-	uint16_t unused[ID_PIPE_NUM];
+	rbtree_type* zone_per_id;
+	/* Array of uint16_t, with ID values.
+	 * unused ID numbers; the first part of the array contains the IDs */
+	uint16_t* unused;
 };
 
 /* create set of tcp connections */
-struct xfrd_tcp_set* xfrd_tcp_set_create(struct region* region, const char *tls_cert_bundle);
+struct xfrd_tcp_set* xfrd_tcp_set_create(struct region* region, const char *tls_cert_bundle, int tcp_max, int tcp_pipeline);
 
 /* init tcp state */
 struct xfrd_tcp* xfrd_tcp_create(struct region* region, size_t bufsize);
@@ -219,6 +244,9 @@ socklen_t xfrd_acl_sockaddr_frm(struct acl_options* acl,
 #endif /* INET6 */
 
 /* create pipeline tcp structure */
-struct xfrd_tcp_pipeline* xfrd_tcp_pipeline_create(region_type* region);
+struct xfrd_tcp_pipeline* xfrd_tcp_pipeline_create(region_type* region,
+	int tcp_pipeline);
+/* pick num uint16_t values, from 0..max-1, store in array */
+void pick_id_values(uint16_t* array, int num, int max);
 
 #endif /* XFRD_TCP_H */
